# escape.manager

CMS da Escape para ser instalado com o Laravel 4. Instalação disponível via Composer.

## Configurações

### composer.json

Edite o seu arquivo `composer.json` adicionando o repositório do escape.manager.

```javascript
    "require": {
        "escapework/manager": "0.9.*"
    },

    "repositories": [
        {
            "type": "composer",
            "url": "http://packages.escape.ppg.br"
        }
    ],
```

Depois, execute o comando:

```sh
$ composer update`
```

### Publicando o pacote e suas configurações

Edite o arquivo `app/config/app.php`, adicionando a seguinte linha no array de `providers`:

```php
    'EscapeWork\Manager\Providers\ManagerServiceProvider',
```

**OBS**: Antes de executar o comando abaixo a base de dados local deve estar configurada.

Depois de baixar o pacote, execute o seguinte comando:

- `$ php artisan manager:configure`;

Caso seja a primeira vez que o comando é executado, ele vai te pedir informações para criação do usuário administrador.

Depois de executar o comando a cima é necessário digitar o seguinte comando:

- `$ php artisan config:publish escapework/manager`

Agora, a URL `seudominio.dev/manager` já está disponível para acesso.

***

## Utilização

### Templates

Para extender o layout master, utilize da seguinte forma:

```php
@extends('manager::layouts.master')

@section('styles')
    @parent
    <link href="{{ asset('css') }}" rel="stylesheet" type="text/css" />
@stop

@section('content')
    Contúdo da página
@stop

@section('scripts')
    @parent
    <script src="{{ asset('file.js') }}" type="text/javascript"></script>
@stop
```

Para editar o menu, crie um template chamado `menu.blade.php` (de preferência, em uma subpasta chamada `manager` dentro da pasta de `views`. Depois, altere a configuração `views.menu`, nas configurações do escape.manager, passando o caminho da view. Ex: `manager.menu`.

```php
<li><a href="{{ route('manager.clientes.index') }}">Clientes</a></li>
<li><a href="{{ route('categorias.index') }}">Categorias</a></li>
```

***

### Permissões de acesso

Para testar as permissões de acesso, utilize o seguinte comando:

```php
Check::can('categories', 'read'); // (page|action) true or false

@if (Check::can('users', 'create'))
    <a href="#" class="btn">Insert new user</a>
@endif
```

Para exibir ou não exibir no menu, pode-se testar como no exemplo abaixo, verificando se o usuário tem pelo menos 1 permissão de acesso a página.

```php
@if (Check::canAny('users')
    <a href="{{ route('usuarios.index') }}" class="btn">Usuários</a>
@endif
```

Você também pode testar a permissão de acesso diretamente no seu controller.

```php
use EscapeWork\Manager\Controllers\ManagerController;

class UserController extends ManagerController
{

    public function __construct()
    {
        # CRUD - default filters
        # create -> methods (create, store)
        # read -> methods (show)
        # update -> methods (edit, update)
        # destroy -> methods (destroy)
        $this->enableDefaultFilters('users');

        # custom permissions filters
        $this->beforeFilter(function()
        {
            Check::canOrFail('users', 'read');
        }, array('only' => array('print')));
    }
}
```

### Templates

Você pode reescrever o template padrão da home. Para isso, basta alterar o array de configurações de templates que fica em `app/config/packages/escapework/manager/config.php`.

```php
    /**
     * Templates que podem ser sobreescritor
     */
    'templates' => array(
        'home'   => 'manager::home',
        'layout' => 'manager::layouts.master'
    ),
```

### CKEditor

Para utilizar qualquer editor com o CKEditor, apenas adicione o paramêtro `data-editor` na sua textarea.

```html
    <textarea data-editor></textarea>
```

### Unit Testing

Para autenticar o usuário, o escapework/manager oferece um TestCase para testar seus controllers do Manager. É só extender a classe `EscapeWork\Manager\TestCase`.

```php
class ProductControllerTest extends \EscapeWork\Manager\TestCase
{

}
```

Também existe um helper para retornos em casos de paginação. Ele contém todos os métodos que o [Illuminate\Pagination\Paginator](http://laravel.com/docs/pagination).

```php
class ProductController extends \EscapeWork\Manager\TestCase
{
    public function test_index()
    {
        $this->product->shouldReceive('paginate')->andReturn($this->paginate);
    }
}
}
```

### Exporter

Existe um CSVExporter para ajudar a exportar arquivos para CSV.

```php
use EscapeWork\Manager\Services\Exporter\CSVExporter;

$exporter = new CSVExporter();
$exporter->writeLine(array('Field 1', 'Field 2', 'Field 3'));
return $exporter->export(array('filename' => 'your-file.csv'));
```

Note que o retorno do método `$exporter->export()` é um objeto `Response` do Laravel com os devidos headers já setados. A melhor opção, é retornar o objeto inteiro de uma ação de um controller para que o arquivo seja exportado.

#### Sobreescrevendo rotas

As seguintes ações de controllers podem ser sobreescritas para as suas próprias:

* `/` - Altere a configuração `manager::controllers.home`;

### Ordernação nas listagens

Para adicionar uma ordenação nas listagens do manager, adicione o método `order_by_link` nas colunas do seu `thead`.

```html
<thead>
    <tr>
        <th><a href="{{ order_by_link('route', 'order_by_field', array('search_params' => 'values')) }}">#</a></th>
        <th><a href="{{ order_by_link('manager.news.index', 'title', Input::all()) }}">Título</a></th>
        <th><a href="{{ order_by_link('manager.news.index', 'order', Input::all()) }}">Ordem</a></th>
    </tr>
</thead>
```

***

#### Dependências e Componentes

O **escape.manager** utiliza os seguintes componentes:

- [EscapeWork/Msg](https://github.com/EscapeWork/Msg);
- [EscapeWork/LaravelHelpers](https://github.com/EscapeWork/LaravelHelpers);
- [Twitter Bootstrap](http://twitter.github.io/bootstrap/);
- [jQuery](http://jquery.com/);
- [jQuery UI](http://jqueryui.com/);
- [jQuery Validate](http://jqueryvalidation.org/);
* [CKEditor](http://ckeditor.com/).

### Alguns projetos utilizando o escapework/manager

* [ADVB](http://www.advb.com.br)
* [Assintecal](http://ww3.assintecal.org.br)
* [Brasil By Bags](http://www.brasilbybags.com)
* [ABIACAV](http://www.abiacav.org.br)
* [FCBB](http://www.fcbybrasil.com)
* [escape.os](http://os.escape.ppg.br)
* [Gabriel Rosa Leilões](http:://gabrielrosaleiloes.com.br)
* escape.commerce

### Módulos para utilizar junto com o escapework/manager

* [escapework/manager-banners](https://bitbucket.org/escapecria/manager-banners) - Controle de banners
* [escapework/manager-subscribers](https://bitbucket.org/escapecria/manager-subscribers) - Controle de contatos de newsletter
* [escapework/manager-calendars](https://bitbucket.org/escapecria/manager-calendars) - Gerenciamento de Agenda
* [escapework/manager-news](https://bitbucket.org/escapecria/manager-news) - Gerenciamento de Notícias
* [escapework/manager-medias](https://bitbucket.org/escapecria/manager-medias) - Controle de mídias
* [escapework/manager-downloads](https://bitbucket.org/escapecria/manager-medias) - Cadastro de downloads
