<?php namespace EscapeWork\Manager\Controllers;

use EscapeWork\Manager\Repositories\User\UserRepository;
use EscapeWork\Manager\Repositories\Role\RoleRepository;
use EscapeWork\Manager\Repositories\UserReminder\UserReminderRepository;
use EscapeWork\Manager\Facades\UserAuth;
use EscapeWork\Manager\Facades\UserPassword;
use View;
use Input;
use Redirect;
use Msg;
use Config;
use Request;
use App;

class UserController extends ManagerController
{

    public function __construct(
        UserRepository $user,
        UserReminderRepository $userReminder,
        RoleRepository $role)
    {
        $this->user         = $user;
        $this->userReminder = $userReminder;
        $this->role         = $role;

        $this->enableDefaultFilters('usuarios');
    }

    public function index()
    {
        $view        = View::make(Config::get('manager::views.users.index'));
        $view->users = $this->user->all();

        return $view;
    }

    public function create()
    {
        Msg::setInfo('Inserindo um novo usuário');

        $this->user->fill(Input::old());

        $view       = View::make(Config::get('manager::views.users.form'));
        $view->user = $this->user;
        $view->role = $this->role;
        $view->edit = false;

        return $view;
    }

    public function store()
    {
        $this->user->fill(Input::all());

        if (! $this->user->validate()) {
            return Redirect::route('manager.users.create')
                    ->withInput()
                    ->withErrors($this->user->getValidationErrors());
        }

        $this->user->save();

        return Redirect::route('manager.users.index')
                ->with('message', 'O usuário <strong>' . $this->user->name . '</strong> foi inserido com sucesso!');
    }

    public function show($id)
    {
        if (Request::ajax()) {
            return $this->user->find($id);
        } else {
            return Redirect::route('manager.users.index');
        }
    }

    public function edit($id)
    {
        $this->user = $this->user->findOrFail($id);

        Msg::setInfo('Editando o usuário <strong>'.$this->user->name.'</strong>');

        $view              = View::make(Config::get('manager::views.users.form'));
        $view->edit        = true;
        $view->user        = $this->user;
        $view->role        = $this->role;

        return $view;
    }

    public function update($id)
    {
        $this->user = $this->user->findOrFail($id);
        $this->user->fill(Input::all());

        if (! $this->user->validate()) {
            return Redirect::route('manager.users.edit', $this->user->id)
                            ->withInput()
                            ->withErrors($this->user->getValidationErrors());
        }

        $this->user->save();

        return Redirect::route('manager.users.index')
                ->with('message', 'O usuário <strong>'. $this->user->name .'</strong> foi editado com sucesso!');
    }

    public function destroy($id)
    {
        $this->user = $this->user->findOrFail($id);

        # verificando se o usuário logado não está tentando se excluir
        if (UserAuth::check() && $this->user->id == UserAuth::user()->id) {
            return Redirect::route('manager.users.index')
                    ->with('error', '<strong>'.$this->user->name.'</strong>, não é possível excluir o seu próprio usuário');
        }

        $this->user->delete();

        return Redirect::route('manager.users.index')
                ->with('message', 'O usuário <strong>'.$this->user->name.'</strong> foi excluído com sucesso!');
    }

    public function sendResetPassword()
    {
        $credentials = array(
            'email' => Input::get('email')
        );

        if (! $user = $this->user->getByEmail($credentials['email'])) {
            return Redirect::route('manager.login')
                    ->with('error', 'Este e-mail não está cadastrado no sistema');
        }

        UserPassword::remind($credentials, function($m)
        {
            $m->subject('Seu lembrete de senha');
        });

        return Redirect::route('manager.login')
            ->with('message', 'Foi enviado para o seu e-mail instruções para refazer a sua senha.');
    }

    public function getResetPassword($token)
    {
        if ( ! $this->userReminder = $this->userReminder->getByToken($token)) {
            return Redirect::route('manager.login')
                        ->with('error', 'Token enviado é incorreto. Verifique o link enviado ao seu e-mail e tente novamente.');
        }

        $view               = View::make('manager::users.reset');
        $view->userReminder = $this->userReminder;
        $view->token        = $token;

        return $view;
    }

    public function postResetPassword()
    {
        $credentials = Input::only('email', 'password', 'password_confirmation', 'token');

        if (! $user = $this->user->getByEmail($credentials['email'])) {
            return Redirect::route('manager.login')
                    ->with('error', 'O e-mail digitado para refazer a sua senha não existe!');
        }

        UserPassword::reset($credentials, function($user, $password)
        {
            $user->password = $password;
            $user->save();
        });

        return Redirect::route('manager.login')
                       ->with('message', 'Sua senha foi alterada com sucesso! Faça o login agora.')
                       ->withInput();
    }
}
