<?php namespace EscapeWork\Manager\Providers;

use EscapeWork\Manager\Repositories\User\EloquentUserRepository;
use EscapeWork\Manager\Repositories\RoleAction\EloquentRoleActionRepository;
use EscapeWork\Manager\Models\User;
use EscapeWork\Manager\Models\RoleAction;
use EscapeWork\Manager\Commands\ConfigureCommand;
use EscapeWork\Manager\Facades\UserAuth;
use EscapeWork\Manager\Permission;
use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        $this->package('escapework/manager', 'manager', realpath(__DIR__ . '/../../..'));

        require_once(__DIR__ . '/../../../routes.php');
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->registerPermission();
        $this->registerCommands();
        $this->bindRepositories();
        $this->callProviders();
        $this->createAliases();
        $this->createConfigurations();
    }

    /**
     * Registering the permission check class
     */
    protected function registerPermission()
    {
        $this->app['escapework.manager.permission'] = $this->app->share(function($app)
        {
            $user = UserAuth::user();

            $permission = $app->make('EscapeWork\Manager\Permission', array($user, new EloquentRoleActionRepository(new RoleAction)));
            $permission->fetchAllowedActions()
                       ->formatAllowedActionsArray();

            return $permission;
        });
    }

    /**
     * Calling the providers needed
     */
    protected function callProviders()
    {
        $this->app->register('EscapeWork\Manager\Providers\UserServiceProvider');

        if ($this->app['request']->is('manager/*')) {
            $this->app->register('EscapeWork\Manager\Providers\ErrorServiceProvider');
        }

        # escapework/msg
        $this->app->register('EscapeWork\Msg\Laravel\MsgServiceProvider');
    }

    /**
     * Binding the repositories
     */
    protected function bindRepositories()
    {
        $this->app->bind('EscapeWork\Manager\Repositories\User\UserRepository', 'EscapeWork\Manager\Repositories\User\EloquentUserRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\UserReminder\UserReminderRepository', 'EscapeWork\Manager\Repositories\UserReminder\EloquentUserReminderRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\Role\RoleRepository', 'EscapeWork\Manager\Repositories\Role\EloquentRoleRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\RoleAction\RoleActionRepository', 'EscapeWork\Manager\Repositories\RoleAction\EloquentRoleActionRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\UserRole\UserRoleRepository', 'EscapeWork\Manager\Repositories\UserRole\EloquentUserRoleRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\Page\PageRepository', 'EscapeWork\Manager\Repositories\Page\EloquentPageRepository');
        $this->app->bind('EscapeWork\Manager\Repositories\PageAction\PageActionRepository', 'EscapeWork\Manager\Repositories\PageAction\EloquentPageActionRepository');

        $this->app->bind('EscapeWork\Manager\Permission', function($app, $parameters)
        {
            return new Permission($parameters[0], $parameters[1]);
        });
    }

    /**
     * Criando os alias
     */
    protected function createAliases()
    {
        $this->app->booting(function()
        {
            $loader = AliasLoader::getInstance();

            $loader->alias('Check', 'EscapeWork\Manager\Facades\Check');
            $loader->alias('Eloquent', 'EscapeWork\LaravelHelpers\Eloquent');
            $loader->alias('Msg', 'EscapeWork\Msg\Msg');
        });
    }

    /**
     * Registrando os comandos
     */
    public function registerCommands()
    {
        $this->app['command.manager.configure'] = $this->app->share(function($app)
        {
            return new ConfigureCommand(new EloquentUserRepository(new User));
        });

        $this->commands('command.manager.configure');
    }

    /**
     * Criando as configurações em tempo de execução
     */
    protected function createConfigurations()
    {
        $this->app['config']->set('manager::packages', array());
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return array('escapework.manager.permission', 'command.manager.configure');
    }
}
