<?php namespace EscapeWork\Manager;

use Mockery as m;

class PermissionTest extends \PHPUnit_Framework_TestCase
{

    public function setUp()
    {
        $this->user       = m::mock('EscapeWork\Manager\Repositories\User\UserRepository');
        $this->user->role = m::mock('EscapeWork\Manager\Repositories\Role\RoleRepository');
        $this->user->role->all_permissions = 0;
        $this->roleAction = m::mock('EscapeWork\Manager\Repositories\RoleAction\RoleActionRepository');
    }

    public function testFetchAllowedActions()
    {
        $this->roleAction->shouldReceive('getUserActions')->once()->andReturn($this->roleAction);
        $this->roleAction->shouldReceive('toArray')->once()->andReturn([]);

        $permission = new Permission($this->user, $this->roleAction);
        $permission->fetchAllowedActions();
        $this->assertInternalType('array', $permission->actions);
    }

    public function testCanWithAuthorizedAccessShouldReturnTrue()
    {
        $permission = new Permission($this->user, $this->roleAction);
        $permission->actions['user']['read'] = true;
        $this->assertTrue($permission->can('user', 'read'));
    }

    public function testCanWithUnauthorizedAccessShouldReturnNull()
    {
        $permission = new Permission($this->user, $this->roleAction);
        $this->assertFalse($permission->can('user', 'read'));
    }

    public function testCanWithAllPermissionsShouldReturnTrue()
    {
        $this->user->role->all_permissions = 1;
        $permission = new Permission($this->user, $this->roleAction);

        $this->assertTrue($permission->can('user', 'read'));
    }

    public function testCanOrFailWithAuthorizedAccessShouldReturnTrue()
    {
        $permission = new Permission($this->user, $this->roleAction);
        $permission->actions['user']['read'] = true;
        $this->assertTrue($permission->canOrFail('user', 'read'));
    }

    /**
     * @expectedException EscapeWork\Manager\Exceptions\UnauthorizedExcpetion
     */
    public function testCanOrFailWithUnauthorizedAccessShouldThrowAnExcpetion()
    {
        $permission = new Permission($this->user, $this->roleAction);
        $permission->canOrFail('user', 'read');
    }

    public function testFormatAllowedActionsArray()
    {
        $permission = new Permission($this->user, $this->roleAction);
        $permission->actions = [
            ['pageSlug' => 'users', 'actionType' => 'read'],
            ['pageSlug' => 'users', 'actionType' => 'destroy'],
            ['pageSlug' => 'pages', 'actionType' => 'read'],
        ];
        $permission->formatAllowedActionsArray();

        $this->assertInternalType('array', $permission->actions);
        $this->assertTrue(isset($permission->actions['users']['read']));
        $this->assertTrue(isset($permission->actions['users']['destroy']));
        $this->assertTrue(isset($permission->actions['pages']['read']));
        $this->assertFalse(isset($permission->actions['pages']['destroy']));
    }
}
